import wx

"""Module contains all user defined events
"""


ID_START = wx.NewId()

ID_NEXTTAB = wx.NewId()
ID_PREVTAB = wx.NewId()
ID_CLOSETAB = wx.NewId()
ID_NEXTPAGE = wx.NewId()
ID_PREVPAGE = wx.NewId()
ID_REFRESHTAB = wx.NewId()
ID_EXIT = wx.NewId()
ID_RESET = wx.NewId()
ID_UNDO = wx.NewId()
ID_REDO = wx.NewId()
ID_CUT = wx.NewId()
ID_COPY = wx.NewId()
ID_PASTE = wx.NewId()
ID_SELALL = wx.NewId()
ID_HELP = wx.NewId()
ID_GOTOHEADER = wx.NewId()
ID_GOTOPANEL = wx.NewId()
ID_GOTOFOOTER = wx.NewId()
ID_GOTODESKTOP = wx.NewId()
ID_SHOWHEADER = wx.NewId()
ID_SHOWPANEL = wx.NewId()
ID_SHOWFOOTER = wx.NewId()
ID_SHOWDESKTOP = wx.NewId()
ID_SHOWTOOLBAR1 = wx.NewId()
ID_SHOWTOOLBAR2 = wx.NewId()
ID_SHOWSTATUSBAR = wx.NewId()
ID_RETVALUE = wx.NewId()

ID_ZOOM = wx.NewId()
ID_PAGE_SOURCE = wx.NewId()
ID_PAGE_BLOCK = wx.NewId()
ID_PAGE_VIEW = wx.NewId()
ID_HISTORY_SHOW = wx.NewId()
ID_CLEAR_HISTORY = wx.NewId()
ID_BOOKMARK_ADD = wx.NewId()
ID_SHOW_BOOKMARKS = wx.NewId()
ID_SHOW_DOWNLOAD = wx.NewId()
ID_DOWNLOAD_OPTIONS = wx.NewId()
ID_GENERAL_OPTIONS = wx.NewId()
ID_DOWNLOAD_OPTIONS = wx.NewId()
ID_FIND = wx.NewId()
ID_REPLACE = wx.NewId()
ID_PRINT = wx.NewId()
ID_PRINT_PREVIEW = wx.NewId()
ID_PRINT_SETTINGS = wx.NewId()
ID_WEB_BACK = wx.NewId()
ID_WEB_FORWARD = wx.NewId()
ID_WEB_STOP = wx.NewId()
ID_WEB_REFRESH = wx.NewId()
ID_WEB_ADDBOOKMARK = wx.NewId()
ID_WEB_NEW_WINDOW = wx.NewId()
ID_WEB_SOURCE = wx.NewId()
ID_WEB_UP = wx.NewId()
ID_WEB_DOWN = wx.NewId()
ID_WEB_EDIT = wx.NewId()
ID_TASKBAR_SHOW = wx.NewId()
ID_TASKBAR_HIDE = wx.NewId()
ID_TASKBAR_CLOSE = wx.NewId()
ID_TASKBAR_TOGGLE = wx.NewId()
ID_LOAD = wx.NewId()
ID_SAVE = wx.NewId()
ID_SAVE_AS = wx.NewId()

ID_END = wx.NewId()
